#pragma implementation
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdarg.h>
#include <pwd.h>
#include <unistd.h>
#include "diajava.h"
#include "diajava.m"
#include "../paths.h"
#include <dialog.h>

#define USE_JAVA	0

static bool dbg_com = false;

PUBLIC DIAJAVA::DIAJAVA(bool guiok)
{
	pop = NULL;
	if (getuid()==0 && getenv("DBG_COM")!=NULL) dbg_com = true;
	/* #Specification: remadmin / user id of the process
		We try to execute remadmin as the real user
		who started linuxconf. This has several advantages.
		One is that the user .Xdefaults is available.
		The other is that remadmin has access to the normal
		.Xauthority file and is allowed to connect to the
		X server even if access control is restrictive (xhost -)
	*/
	char pipestr[100],textstr[100];
	sprintf (pipestr,"--pipe %s",popen_SOCKETHOLDER);
	sprintf (textstr,"--text --pipe %s",popen_SOCKETHOLDER);
	pop = new POPEN ("remadmin"
		,guiok ? pipestr : textstr
		,popen_getloginuid());
	if (!pop->isok()){
		delete pop;
		pop = NULL;
	}
}

/*
	When talking to the user interface server using a socket
	(this is happenning when connecting from a browser)
*/
PUBLIC DIAJAVA::DIAJAVA(int handlein, int handleout, const char *lines)
{
	pop = new POPENFD (handlein,handleout);
	pop->loadout (lines);
}

PUBLIC DIAJAVA::~DIAJAVA()
{
	delete pop;
}
/*
	Return != if the java user interface server is correctly started
*/
PUBLIC int DIAJAVA::is_ok()
{
	return pop != NULL;
}


PUBLIC ITEM_VAL::ITEM_VAL(const char *_id, const char *_val)
{
	id.setfrom (_id);
	val.setfrom (_val);
}

PUBLIC ITEM_VAL *ITEM_VALS::getitem(int no)
{
	return (ITEM_VAL*)ARRAY::getitem(no);
}

/*
	This function should return NULL if the ID is not found.
	But this should never happen.
*/
PUBLIC const char *ITEM_VALS::getval (const char *id)
{
	const char *ret = "";
	int n = getnb();
	for (int i=0; i<n; i++){
		ITEM_VAL *v = getitem(i);
		if (v->id.cmp(id)==0){
			ret = v->val.get();
			break;
		}
	}
	return ret;
}
PUBLIC const char *DIAJAVA::getval (const char *id)
{
	return vals.getval (id);
}

bool diajava_treemenu=false;

PUBLIC void DIAJAVA::wait (
	char dianame[200],	// Name of the dialog where the event occured
	char actionid[100],	// Parameter associated with the event
	int &menu,		// Will contain the menu line selected or -1
	int &but)		// Will contain the button selected or -1
{
	menu = but = -1;
	dianame[0] = actionid[0] = '\0';
	vals.remove_all();
	bool done = false;
	bool first = true;
	while (!done){
		if (!first){	// Let process the html body
			if (pop->wait (10000000)==-1){
				fprintf (stderr,MSG_U(E_REMEXIT,"remadmin (GUI frontend) exiting abnormaly\n"));
				exit(-1);
			}
		}
		first = false;
		char buf[1000];
		int nb;
		while ((nb = pop->readout(buf,sizeof(buf)-1))!=-1){
			strip_end (buf);
			if (dbg_com) fprintf (stderr,"readout :%s:\n",buf);
			char verb[20];
			char *pt = str_copyword (verb,buf,sizeof(verb)-1);
			if (strcmp(verb,"action")==0){
				pt = str_copyword (dianame,pt,200-1);
				str_copyword (actionid,pt,100-1);
				if (actionid[0] != '\0'){
					if (strcmp(actionid,"badver")==0){
						fprintf (stderr,MSG_U(E_REMVER
							,"Incompatible version of the GUI frontend\n"));
						exit(-1);
					}
					int no = atoi(actionid+1);
					if (actionid[0] == 'B'){
						but = no;
					}else if (actionid[0] == 'M'){
						menu = no;
					}
					done = true;
				}
			}else if (strcmp(verb,"dump")==0){
				char id[100];
				pt = str_copyword (dianame,pt,200-1);
				pt = str_copyword (id,pt,sizeof(id)-1);
				if (id[0] != '\0'){
					vals.add (new ITEM_VAL(id,pt+1));
				}
			}else if (strcmp(verb,"prefer")==0){
				while (1){
					char pref[100];
					pt = str_copyword (pref,pt,sizeof(pref)-1);
					if (pref[0] == '\0') break;
					if (strcmp(pref,"html")==0){
						diagui_sethtmlhelp();
					}else if (strcmp(pref,"nogfx")==0){
						menubox_setnogfx();
					}else if (strcmp(pref,"modal")==0){
						uithread_setmodal();
					}else if (strcmp(pref,"treemenu")==0){
						diajava_treemenu = true;
					}
				}				
				done = true;
				strcpy (actionid,"prefer");
				break;
			}else if (strcmp(verb,"gethtml")==0){
				char path[PATH_MAX];
				str_copyword (path,pt,PATH_MAX-1);
				diagui_sendhtmlhelp (path);
			}else if (strcmp(verb,"ncurses")==0){
				done = true;
				strcpy (actionid,"ncurses");
				// No front-end, treemenu can deal with the ncurse interface
				diajava_treemenu = true;
				break;
			}else{
				fprintf (stderr,"err :%s:\n",buf);
			}
		}
		while ((nb = pop->readerr(buf,sizeof(buf)-1)) != -1){
			fprintf (stderr,MSG_U(E_REMERR,"Error message from remadmin :%s"),buf);
		}
	}
}

PUBLIC void DIAJAVA::send (const char *ctl, ...)
{
	char buf[1000];
	va_list list;
	va_start (list,ctl);
	vsnprintf (buf,sizeof(buf)-1,ctl,list);
	pop->send (buf);
	va_end (list);
}

PUBLIC void DIAJAVA::sendcmd (int cmd, const char *ctl, ...)
{
	char buf[1000];
	int n = sprintf (buf,"%d ",cmd);
	va_list list;
	va_start (list,ctl);
	vsprintf (buf+n,ctl,list);
	pop->send (buf);
	va_end (list);
}

