#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <misc.h>
#include "internal.h"
#include "../paths.h"
#include "apache.m"
#include <userconf.h>
#include <subsys.h>
#include <translat.h>


static APACHE_HELP_FILE help_httpd ("httpd");

static const char subsys_httpd[]="httpd";
static LINUXCONF_SUBSYS subb (subsys_httpd,P_MSG_R(M_APACHE));


CONFIG_FILE httpd_conf ("/etc/httpd/conf/httpd.conf",help_httpd
		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED,subsys_httpd);

static PRIVILEGE p_apache ("apache"
	,P_MSG_U(T_PRIVIAPACHE,"Apache administration")
	,P_MSG_U(T_SERVICES,"1-Services"));

/* #Specification: httpd config / principle
	Linuxconf supports the apache httpd server. While the apache server
	share most of its configuration file format with ncsa httpd, I have not
	made any attempt at "virtualizing" linuxconf. Linuxconf hope to really
	support Apache closer and closer.

	It is expect that linuxconf will grow into something more modular one
	day and modules will be available to manage other things than apache
	if this is needed.
*/

static const char HTTPD[]="HTTPD";
static const char PORT[]="Port";
#if 0
	static const char USER[]="User";
	static const char GROUP[]="Group";
#endif
static const char SERVERTYPE[]="ServerType";
static const char PIDFILE[]="PidFile";
static const char STARTSERVERS[]="StartServers";
static const char MAXCLIENTS[]="MaxClients";
static const char MAXREQUESTSPERCHILD[]="MaxRequestsPerChild";
static const char VIRTUALHOST[]="VirtualHost";
static const char TIMEOUT[]="Timeout";
static const char MINSPARESERVERS[]="MinSpareServers";
static const char MAXSPARESERVERS[]="MaxSpareServers";
static const char SCOREBOARDFILE[]="ScoreBoardFile";
static const char KEEPALIVE[]="KeepAlive";
static const char KEEPALIVETIMEOUT[]="KeepAliveTimeout";

static const char SERVERADMIN[]="ServerAdmin";
static const char SERVERNAME[]="ServerName";
static const char SERVERALIAS[]="ServerAlias";
static const char SERVERROOT[]="ServerRoot";
static const char DOCUMENTROOT[]="DocumentRoot";
static const char ERRORLOG[]="ErrorLog";
static const char REFERERLOG[]="RefererLog";
static const char TRANSFERLOG[]="TransferLog";
static const char AGENTLOG[]="AgentLog";
static const char DIRECTORY[]="Directory";
static const char DIRECTORYMATCH[]="Directorymatch";
static const char INDEXES[]="Indexes";
static const char FOLLOWSYMLINKS[]="FollowSymlinks";
static const char ALLOWOVERRIDE[]="AllowOverride";
static const char HOSTNAMELOOKUPS[]="HostnameLookups";
static const char EXECCGI[]="ExecCgi";
static const char INCLUDES[]="Includes";
static const char INCLUDESNOEXEC[]="IncludesNOEXEC";
static const char MULTIVIEWS[]="MultiViews";
static const char SYMLINKSIFOWNERMATCH[]="SymLinksIfOwnerMatch";
static const char OPTIONS[]="Options";
static const char ALL[]="All";
static const char NONE[]="None";
static const char AUTHCONFIG[]="AuthConfig";
static const char FILEINFO[]="FileInfo";
static const char LIMIT[]="Limit";
static const char SCRIPTALIAS[]="ScriptAlias";
static const char SETUIDUSER[]="User";
static const char SETUIDGROUP[]="Group";



PUBLIC HTTPD_DOMAIN *HTTPD_DOMAINS::getitem(int no)
{
	return (HTTPD_DOMAIN*)ARRAY::getitem(no);
}

static void writeif (
	FILE *fout,
	const char *prefix,
	const char *title,
	CSSTRING &s)
{
	if (!s.is_empty()){
		comment_write (s.comment,fout);
		fprintf (fout,"%s%s %s\n",prefix,title,s.get());
	}
}


PUBLIC void HTTPD_FEATURES::setallopt(int val)
{
	char cval = (val != 0) ? 1 : 0;
	symlinksifownermatch = cval;
	execcgi = cval;
	multiviews = cval;
	includes = cval;
	includesnoexec = cval;
	indexes=cval;
	followsymlinks=cval;
}

PUBLIC HTTPD_FEATURES::HTTPD_FEATURES()
{
	setallopt (0);
}

PUBLIC void HTTPD_OVERRIDE::write (const char *prefix, FILE *fout)
{
	char buf[1000];
	int len = sprintf (buf,"%s%s",prefix,ALLOWOVERRIDE);
	int startlen = len;
	if (authconfig) len += sprintf (buf+len," %s",AUTHCONFIG);
	if (fileinfo) len += sprintf (buf+len," %s",FILEINFO);
	if (indexes) len += sprintf (buf+len," %s",INDEXES);
	if (limit) len += sprintf (buf+len," %s",LIMIT);
	if (options) len += sprintf (buf+len," %s",OPTIONS);

	if (startlen != len) fprintf (fout,"%s\n",buf);
}

PUBLIC void HTTPD_FEATURES::write (const char *prefix, FILE *fout)
{
	char buf[1000];
	int len = sprintf (buf,"%s%s",prefix,OPTIONS);
	int startlen = len;
	if (execcgi) len += sprintf (buf+len," %s",EXECCGI);
	if (includes) len += sprintf (buf+len," %s",INCLUDES);
	if (includesnoexec) len += sprintf (buf+len," %s",INCLUDESNOEXEC);
	if (multiviews) len += sprintf (buf+len," %s",MULTIVIEWS);
	if (indexes) len += sprintf (buf+len," %s",INDEXES);
	if (followsymlinks) len += sprintf (buf+len," %s",FOLLOWSYMLINKS);
	if (symlinksifownermatch) len += sprintf (buf+len," %s",SYMLINKSIFOWNERMATCH);

	if (startlen != len) fprintf (fout,"%s\n",buf);
}

PUBLIC HTTPD_DIR::HTTPD_DIR (const char *path)
{
	this->path.setfrom (path);
}
PUBLIC HTTPD_DIR::HTTPD_DIR ()
{
}

PUBLIC void HTTPD_DIR::write (FILE *fout)
{
	fprintf (fout,"<%s %s>\n",DIRECTORY,path.get());
	feats.write ("\t",fout);
	over.write("\t",fout);
	fprintf (fout,"</%s>\n",DIRECTORY);
}

PUBLIC void HTTPD_DIRS::write (FILE *fout)
{
	int n = getnb();
	for (int i=0; i<n; i++){
		getitem(i)->write (fout);
	}
}

PUBLIC void HTTPD_DOMOPT::write(const char *prefix, FILE *fout)
{
	writeif (fout,prefix,SERVERADMIN,serveradmin);
	writeif (fout,prefix,SERVERNAME,servername);
	writeif (fout,prefix,SERVERALIAS,serveralias);
	writeif (fout,prefix,DOCUMENTROOT,documentroot);
	writeif (fout,prefix,ERRORLOG,errorlog);
	writeif (fout,prefix,REFERERLOG,refererlog);
	writeif (fout,prefix,TRANSFERLOG,transferlog);
	writeif (fout,prefix,AGENTLOG,agentlog);
	writeif (fout,prefix,SCRIPTALIAS,scriptalias);
	writeif (fout,prefix,SETUIDUSER,setuiduser);
	writeif (fout,prefix,SETUIDGROUP,setuidgroup);	feats.write (prefix,fout);
	for (int i=0; i<tbunknown.getnb(); i++){
		CSSTRING *cs = (CSSTRING*)tbunknown.getitem(i);
		comment_write (cs->comment,fout);
		fprintf (fout,"%s%s\n",prefix,cs->get());
	}
}

PUBLIC HTTPD_DOMAIN::HTTPD_DOMAIN()
{
}
PUBLIC HTTPD_DOMAIN::HTTPD_DOMAIN(const char *domain)
{
	name.setfrom (domain);
}
PUBLIC void HTTPD_DOMAIN::write(FILE *fout)
{
	fprintf (fout,"<%s %s>\n",VIRTUALHOST,name.get());
	opt.write ("\t",fout);
	dirs.write (fout);
	fprintf (fout,"</%s>\n",VIRTUALHOST);
}

PUBLIC HTTPD_CONFIG::HTTPD_CONFIG()
{
	port = 80;
	startservers = 5;
	maxclients = 150;
	maxrequestsperchild = 30;
	minspareservers = 5;
	maxspareservers = 10;
	keepalive = 5;
	keepalive_timeout = 5;
	servertype = 0;
	hostnamelookups = 0;
	pidfile.setfrom ("logs/httpd.pid");
	#if 0
		user.setfrom ("nobody");
		group.setfrom ("nobody");
	#endif
	defdom.setuiduser.setfrom ("nobody");
	defdom.setuidgroup.setfrom ("nobody");
	timeout = 400;
	scoreboardfile.setfrom ("logs/apache_runtime_status");
	FILE *fin = httpd_conf.fopen ("r");
	if (fin != NULL){
		char buf[3000];
		SSTRING comments;
		HTTPD_DOMOPT *dom = &defdom;
		HTTPD_DIRS *ptdirs = &dirs;
		HTTPD_DIR *ptdir = NULL;
		HTTPD_FEATURES *ptfeats = &dom->feats;
		while (fgets_comments(buf,sizeof(buf)-1,fin,comments)!=NULL){
			char *pt = str_skip (buf);
			if (*pt == '<'){
				pt++;
				bool endscope = false;
				if (*pt == '/'){
					endscope = true;
					pt++;
				}
				char *end = strchr (pt,'>');
				if (end != NULL) *end = '\0';
				char word[200],word2[200];
				char *arg = str_copyword (word,pt);
				str_copyword (word2,arg);
				if (stricmp(word,VIRTUALHOST)==0){
					if (endscope){
						dom = &defdom;
						ptdirs = &dirs;
					}else{
						HTTPD_DOMAIN *newdom = new HTTPD_DOMAIN (word2);
						domains.add (newdom);
						dom = &newdom->opt;
						ptdirs = &newdom->dirs;
					}
					ptfeats = &dom->feats;
				}else if (stricmp(word,DIRECTORY)==0){
					if (endscope){
						dom = &defdom;
						ptdirs = &dirs;
						ptfeats = &dom->feats;
						ptdir = NULL;
					}else{
						ptdir = new HTTPD_DIR (word2);
						ptdirs->add (ptdir);
						ptfeats = &ptdir->feats;
					}
				}else{
					xconf_error (MSG_U(E_UNKNOWNDIR,"Unknown scope directive: %s")
						,word);
				}
			}else{
				char word[200];
				const char *arg = str_copyword (word,pt);
				arg = str_skip(arg);
				if (stricmp(word,SERVERADMIN)==0){
					dom->serveradmin.setcomment(comments);
					dom->serveradmin.setfrom(arg);
				}else if (stricmp(word,SERVERNAME)==0){
					dom->servername.setcomment(comments);
					dom->servername.setfrom(arg);
				}else if (stricmp(word,SERVERALIAS)==0){
					dom->serveralias.setcomment(comments);
					dom->serveralias.setfrom(arg);
				}else if (stricmp(word,DOCUMENTROOT)==0){
					dom->documentroot.setcomment(comments);
					dom->documentroot.setfrom(arg);
				}else if (stricmp(word,ERRORLOG)==0){
					dom->errorlog.setcomment(comments);
					dom->errorlog.setfrom(arg);
				}else if (stricmp(word,REFERERLOG)==0){
					dom->refererlog.setcomment(comments);
					dom->refererlog.setfrom(arg);
				}else if (stricmp(word,TRANSFERLOG)==0){
					dom->transferlog.setcomment(comments);
					dom->transferlog.setfrom(arg);
				}else if (stricmp(word,AGENTLOG)==0){
					dom->agentlog.setcomment(comments);
					dom->agentlog.setfrom(arg);
				}else if (stricmp(word,SCRIPTALIAS)==0){
					dom->scriptalias.setcomment(comments);
					dom->scriptalias.setfrom(arg);
				}else if (stricmp(word,SETUIDUSER)==0){
					dom->setuiduser.setcomment(comments);
					dom->setuiduser.setfrom(arg);
				}else if (stricmp(word,SETUIDGROUP)==0){
					dom->setuidgroup.setcomment(comments);
					dom->setuidgroup.setfrom(arg);
 				}else if (stricmp(word,ALLOWOVERRIDE)==0){
					if (ptdir == NULL){
						xconf_error ("allow ptdir ==NULL");
					}else{
						HTTPD_OVERRIDE *ptover = &ptdir->over;
						while (1){
							arg=str_copyword(word,arg);
							if (word[0] == '\0'){
								break;
							}else if (stricmp(word,NONE)==0){
								ptover->setallopt(0);
							}else if (stricmp(word,ALL)==0){
								ptover->setallopt(1);
							}else if (stricmp(word,AUTHCONFIG)==0){
								ptover->authconfig = 1;
							}else if (stricmp(word,FILEINFO)==0){
								ptover->fileinfo = 1;
							}else if (stricmp(word,INDEXES)==0){
								ptover->indexes = 1;
							}else if (stricmp(word,LIMIT)==0){
								ptover->limit = 1;
							}else if (stricmp(word,OPTIONS)==0){
								ptover->options = 1;
							}
						}
					}
				}else if (stricmp(word,PORT)==0){
					comment_port.setfrom (comments);
					port = atoi(arg);
				#if 0
					}else if (stricmp(word,USER)==0){
						user.setcomment(comments);
						user.setfrom (arg);
					}else if (stricmp(word,GROUP)==0){
						group.setcomment(comments);
						group.setfrom (arg);
				#endif
				}else if (stricmp(word,SERVERTYPE)==0){
					comment_servertype.setfrom (comments);
					servertype = stricmp(arg,"standalone") == 0 ? 0 : 1;
				}else if (stricmp(word,SERVERROOT)==0){
					serverroot.setcomment (comments);
					serverroot.setfrom (arg);
				}else if (stricmp(word,PIDFILE)==0){
					pidfile.setcomment(comments);
					pidfile.setfrom (arg);
				}else if (stricmp(word,STARTSERVERS)==0){
					comment_startservers.setfrom (comments);
					startservers = atoi(arg);
				}else if (stricmp(word,MAXCLIENTS)==0){
					comment_maxclients.setfrom (comments);
					maxclients = atoi(arg);
				}else if (stricmp(word,MAXREQUESTSPERCHILD)==0){
					comment_maxrequestsperchild.setfrom (comments);
					maxrequestsperchild = atoi(arg);
				}else if (stricmp(word,TIMEOUT)==0){
					comment_timeout.setfrom (comments);
					timeout = atoi(arg);
				}else if (stricmp(word,MINSPARESERVERS)==0){
					comment_minspareservers.setfrom (comments);
					minspareservers = atoi(arg);
				}else if (stricmp(word,MAXSPARESERVERS)==0){
					comment_maxspareservers.setfrom (comments);
					maxspareservers = atoi(arg);
				}else if (stricmp(word,KEEPALIVE)==0){
					comment_keepalive.setfrom (comments);
					keepalive = atoi(arg);
				}else if (stricmp(word,KEEPALIVETIMEOUT)==0){
					comment_keepalive_timeout.setfrom (comments);
					keepalive_timeout = atoi(arg);
				}else if (stricmp(word,SCOREBOARDFILE)==0){
					scoreboardfile.setcomment (comments);
					scoreboardfile.setfrom (arg);
				}else if (stricmp(word,HOSTNAMELOOKUPS)==0){
					comment_hostnamelookups.setfrom (comments);
					hostnamelookups = stricmp (arg,"on")==0;
				}else if (stricmp(word,OPTIONS)==0){
					while (1){
						arg=str_copyword(word,arg);
						if (word[0] == '\0'){
							break;
						}else if (stricmp(word,ALL)==0){
							ptfeats->setallopt (1);
						}else if (stricmp(word,NONE)==0){
							ptfeats->setallopt (0);
						}else if (stricmp(word,INDEXES)==0){
							ptfeats->indexes = 1;
						}else if (stricmp(word,MULTIVIEWS)==0){
							ptfeats->multiviews = 1;
						}else if (stricmp(word,FOLLOWSYMLINKS)==0){
							ptfeats->followsymlinks = 1;
						}else if (stricmp(word,SYMLINKSIFOWNERMATCH)==0){
							ptfeats->symlinksifownermatch = 1;
						}else if (stricmp(word,INCLUDES)==0){
							ptfeats->includes = 1;
						}else if (stricmp(word,INCLUDESNOEXEC)==0){
							ptfeats->includesnoexec = 1;
						}else if (stricmp(word,EXECCGI)==0){
							ptfeats->execcgi = 1;
						}
					}
				}else{
					CSSTRING *cs = new CSSTRING;
					cs->setcomment (comments);
					cs->setfrom (buf);
					dom->tbunknown.add (cs);
					//xconf_error ("Invalid line %s\n",buf);
				}
			}
		}
		fclose (fin);

	}
}

PUBLIC int HTTPD_CONFIG::write ()
{
	int ret = -1;
	FILE *fout = httpd_conf.fopen (&p_apache,"w");
	if (fout != NULL){
		comment_write (comment_servertype,fout);
		fprintf (fout,"%s %s\n",SERVERTYPE,servertype ? "inetd" : "standalone");
		comment_write (comment_port,fout);
		fprintf (fout,"%s %d\n",PORT,port);
		comment_write (comment_startservers,fout);
		fprintf (fout,"%s %d\n",STARTSERVERS,startservers);
		comment_write (comment_minspareservers,fout);
		fprintf (fout,"%s %d\n",MINSPARESERVERS,minspareservers);
		comment_write (comment_maxspareservers,fout);
		fprintf (fout,"%s %d\n",MAXSPARESERVERS,maxspareservers);
		comment_write (comment_keepalive,fout);
		fprintf (fout,"%s %d\n",KEEPALIVE,keepalive);
		comment_write (comment_keepalive_timeout,fout);
		fprintf (fout,"%s %d\n",KEEPALIVETIMEOUT,keepalive_timeout);
		comment_write (comment_maxclients,fout);
		fprintf (fout,"%s %d\n",MAXCLIENTS,maxclients);
		comment_write (comment_maxrequestsperchild,fout);
		fprintf (fout,"%s %d\n",MAXREQUESTSPERCHILD,maxrequestsperchild);
		comment_write (pidfile.comment,fout);
		fprintf (fout,"%s %s\n",PIDFILE,pidfile.get());
		comment_write (comment_timeout,fout);
		fprintf (fout,"%s %d\n",TIMEOUT,timeout);
		#if 0
			comment_write (user.comment,fout);
			fprintf (fout,"%s %s\n",USER,user.get());
			comment_write (group.comment,fout);
			fprintf (fout,"%s %s\n",GROUP,group.get());
		#endif
		comment_write (scoreboardfile.comment,fout);
		fprintf (fout,"%s %s\n",SCOREBOARDFILE,scoreboardfile.get());
		comment_write (serverroot.comment, fout);
		fprintf (fout,"%s %s\n",SERVERROOT,serverroot.get());

		comment_write (comment_hostnamelookups, fout);
		fprintf (fout,"%s %s\n",HOSTNAMELOOKUPS
			,hostnamelookups ? "on" : "off");


		defdom.write ("",fout);
		fprintf (fout,"\n");
		int i;
		for (i=0; i<dirs.getnb(); i++){
			dirs.getitem(i)->write(fout);
			fputs ("\n",fout);
		}
		for (i=0; i<domains.getnb(); i++){
			domains.getitem(i)->write(fout);
			fputs ("\n",fout);
		}
		ret = fclose (fout);
	}
	return ret;
}




