#pragma interface
#ifndef INTERNAL_H
#define INTERNAL_H

#ifndef MISC_H
	#include <misc.h>
#endif
#ifndef MODULE_H
	#include <module.h>
#endif

#ifndef NETCONF_H
	#include <netconf.h>
#endif
#ifndef DAEMONI_H
	#include <daemoni.h>
#endif

#include <modapi_def.h>
#include <modapi.h>

class APACHE_HELP_FILE: public HELP_FILE{
	/*~PROTOBEG~ APACHE_HELP_FILE */
public:
	APACHE_HELP_FILE (const char *fname);
	/*~PROTOEND~ APACHE_HELP_FILE */
};


class MODULE_APACHE: public LINUXCONF_MODULE{
	/*~PROTOBEG~ MODULE_APACHE */
public:
	MODULE_APACHE (void);
	int domenu (MENU_CONTEXT context,
		 const char *key);
	int execmain (int argc, char *argv[]);
	int probe (int , int , bool);
	void setmenu (DIALOG&dia, MENU_CONTEXT context);
	void usage (SSTRINGS&tb);
	/*~PROTOEND~ MODULE_APACHE */
};

class HTTPD_FEATURES{
public:
	char indexes;
	char followsymlinks;
	char includes;
	char execcgi;
	char symlinksifownermatch;
	char multiviews;
	char includesnoexec;

	/*~PROTOBEG~ HTTPD_FEATURES */
public:
	HTTPD_FEATURES (void);
	void setallopt (int val);
	void setdia (DIALOG&dia);
	void write (const char *prefix, FILE *fout);
	/*~PROTOEND~ HTTPD_FEATURES */
};

class HTTPD_OVERRIDE{
public:
	char authconfig;
	char fileinfo;
	char indexes;
	char limit;
	char options;
	/*~PROTOBEG~ HTTPD_OVERRIDE */
public:
	HTTPD_OVERRIDE (void);
	void setallopt (int val);
	void setdia (DIALOG&dia);
	void write (const char *prefix, FILE *fout);
	/*~PROTOEND~ HTTPD_OVERRIDE */
};

class HTTPD_DIR: public ARRAY_OBJ{
public:
	SSTRING path;
	HTTPD_FEATURES feats;
	HTTPD_OVERRIDE over;
	/*~PROTOBEG~ HTTPD_DIR */
public:
	HTTPD_DIR (const char *path);
	HTTPD_DIR (void);
	int edit (void);
	void write (FILE *fout);
	/*~PROTOEND~ HTTPD_DIR */
};
class HTTPD_CONFIG;

class HTTPD_DIRS: public ARRAY{
	/*~PROTOBEG~ HTTPD_DIRS */
public:
	int edit (HTTPD_CONFIG&_httpd);
	HTTPD_DIR *getitem (int no)const;
	void write (FILE *fout);
	int write (void);
	/*~PROTOEND~ HTTPD_DIRS */
};

class HTTPD_DOMOPT{
public:
	//CSSTRING host;
	CSSTRING serveradmin;
	CSSTRING servername;
	CSSTRING serveralias;
	CSSTRING documentroot;
	CSSTRING errorlog;
	CSSTRING transferlog;
	CSSTRING agentlog;
	CSSTRING refererlog;
	CSSTRING scriptalias;
	CSSTRING setuiduser;
	CSSTRING setuidgroup;
	HTTPD_FEATURES feats;
	SSTRINGS tbunknown;		// Directives not managed by this module
	/*~PROTOBEG~ HTTPD_DOMOPT */
public:
	void setdia (DIALOG&dia, bool advanced);
	void write (const char *prefix, FILE *fout);
	/*~PROTOEND~ HTTPD_DOMOPT */
};

class HTTPD_DOMAIN: public ARRAY_OBJ{
public:
	SSTRING name;
	HTTPD_DOMOPT opt;
	HTTPD_DIRS dirs;
	/*~PROTOBEG~ HTTPD_DOMAIN */
public:
	HTTPD_DOMAIN (const char *domain);
	HTTPD_DOMAIN (void);
	int createroot (void);
	int edit (void);
	void write (FILE *fout);
	/*~PROTOEND~ HTTPD_DOMAIN */
};

class HTTPD_DOMAINS: public ARRAY{
	/*~PROTOBEG~ HTTPD_DOMAINS */
public:
	int edit (HTTPD_CONFIG&_httpd);
	HTTPD_DOMAIN *getitem (int no);
	int write (void);
	/*~PROTOEND~ HTTPD_DOMAINS */
};
class HTTPD_CONFIG{
public:
	int in_use;			// Httpd is operationnal on this system
	SSTRING comment_port;
	int port;
	SSTRING comment_startservers;
	int startservers;
	SSTRING comment_maxclients;
	int maxclients;
	SSTRING comment_maxrequestsperchild;
	int maxrequestsperchild;
	SSTRING comment_minspareservers;
	int minspareservers;
	SSTRING comment_maxspareservers;
	int maxspareservers;
	SSTRING comment_servertype;
	char servertype;	// 0: Standalone
						// 1: inetd
	SSTRING comment_keepalive;
	int keepalive;
	SSTRING comment_keepalive_timeout;
	int keepalive_timeout;

	SSTRING comment_hostnamelookups;
	char hostnamelookups;


	CSSTRING serverroot;
	CSSTRING pidfile;
	#if 0
		CSSTRING user;
		CSSTRING group;
	#endif
	HTTPD_DOMAINS domains;
	HTTPD_DOMOPT defdom;		// Option of the main domain
	HTTPD_DIRS dirs;
	SSTRING comment_timeout;
	int timeout;
	CSSTRING scoreboardfile;
	/*~PROTOBEG~ HTTPD_CONFIG */
public:
	HTTPD_CONFIG (void);
	int editdefaults (void);
	HTTPD_DOMAIN *locatevhost (const char *vhost);
	int write (void);
	/*~PROTOEND~ HTTPD_CONFIG */
};

class DAEMON_HTTPD: public DAEMON_INTERNAL{
	/*~PROTOBEG~ DAEMON_HTTPD */
	/*~PROTOEND~ DAEMON_HTTPD */
};

#include "apache.p"

#endif

