#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "internal.h"
#include "dnsconf.m"

extern CONFIG_FILE f_conf;

PUBLIC BIND8_LEXPARSE::BIND8_LEXPARSE (CONFIG_FILE &_cfg)
	: cfg (_cfg)
{
	tbf[0].fin = cfg.fopen ("r");
	tbf[0].path.setfrom (cfg.getpath());
	tbf[0].noline = 0;
	nofile = 0;
	buf[0] = '\0';
	ptbuf = buf;
	err = false;
}

PUBLIC BIND8_LEXPARSE::~BIND8_LEXPARSE ()
{
	for (int i=nofile; i>=0; i--){
		if (tbf[0].fin != NULL) fclose (tbf[i].fin);
	}
}

PUBLIC bool BIND8_LEXPARSE::isok()
{
	return tbf[0].fin != NULL;
}	

PRIVATE int BIND8_LEXPARSE::fillbuf()
{
	int ret = -1;
	while (nofile >= 0){
		if (fgets(buf,sizeof(buf)-1,tbf[nofile].fin)!=NULL){
			tbf[nofile].noline++;
			ptbuf = buf;
			ret = 0;
			break;
		}else{
			fclose (tbf[nofile].fin);
			nofile--;
		}
	}
	return ret;
}
/*
	Is this a special lexical character
*/
static bool bind8_isspecial(char car)
{
	return car == '{' || car == ';' || car == '}';
}
/*
	Retrieve one token from the named.conf file
*/
PUBLIC const char *BIND8_LEXPARSE::gettoken()
{
	const char *ret = NULL;
	while (1){
		const char *start = ptbuf;
		ptbuf = str_skip(ptbuf);
		while (ptbuf[0] == '\0'){
			if (fillbuf()==-1) return NULL;
			start = buf;
			ptbuf = str_skip(ptbuf);
		}
		char car = ptbuf[0];
		if (car == '/' && ptbuf[1] == '/'){
			comment.append (start);
			if (fillbuf()==-1) return NULL;
		}else if (car == '/' && ptbuf[1] == '*'){
			while (1){
				if (ptbuf[0] == '\0'){
					comment.append (start);
					if (fillbuf()==-1) return NULL;
					start = buf;
				}else if (ptbuf[0] == '*' && ptbuf[1] == '/'){
					ptbuf[0] = '\0';
					comment.append (start);
					comment.append ("*/");
					ptbuf += 2;
					break;
				}else{
					ptbuf++;
				}
			}
		}else{
			if (bind8_isspecial(car)){
				token[0] = *ptbuf++;
				token[1] = '\0';
			}else if (car == '"'){
				ptbuf = str_copyquote(token,ptbuf);
			}else{
				char *pt = token;
				while (*ptbuf > ' '	&& !bind8_isspecial(*ptbuf)){
					*pt++ = *ptbuf++;
				}
				*pt = '\0';
			}
			if (strcmp(token,"include")==0){
				const char *incl = getarg1();
				if (incl == NULL){
					break;
				}else{
					if (nofile < (int)(sizeof(tbf)/sizeof(tbf[0]))){
						nofile++;
						char abspath[PATH_MAX];
						context_setabspath (incl,abspath);
						tbf[nofile].path.setfrom (abspath);
						tbf[nofile].fin = fopen (abspath,"r");
						tbf[nofile].noline = 0;
						fillbuf();
					}
				}
			}else{
				ret = token;
				break;
			}
		}
	}
	return ret;
}

PUBLIC void BIND8_LEXPARSE::error(const char *ctl, ...)
{
	char errbuf[1000];
	va_list list;
	va_start (list,ctl);
	vsnprintf (errbuf,sizeof(errbuf)-1,ctl,list);
	va_end (list);
	xconf_error (MSG_U(E_BIND8LEX,"%s\nfile %s line %d")
		,errbuf,tbf[nofile].path.get(),tbf[nofile].noline);
	err = true;
}

/*
	Get the next token and make sure it is followed by a ;
	Signal an error and return NULL if not.
*/
PUBLIC const char *BIND8_LEXPARSE::getarg1()
{
	const char *ret = NULL;
	const char *tok = gettoken();
	if (tok != NULL){
		char tmp[sizeof(token)];
		strcpy (tmp,tok);
		tok = gettoken();
		if (strcmp(tok,";")==0){
			strcpy (token,tmp);
			ret = token;
		}else{
			error (MSG_U(E_EXPECTARG1,"Expect argument followed by a semi-colon"));
		}
	}
	return ret;
}
/*
	Get the next token and make sure it is followed by a ;
	Assume it is either a yes or a no.
	Return true or false for yes and no.
*/
PUBLIC bool BIND8_LEXPARSE::getarg_yesno()
{
	bool ret = false;
	const char *tok = getarg1();
	if (tok != NULL && strcmp(tok,"yes")==0) ret = true;
	return ret;
}

/*
	Return true if there was at least one error
*/
PUBLIC bool BIND8_LEXPARSE::waserr()
{
	return err;
}

/*
	Check if the next token is specific string.
	Signal an error if not.
	Return -1 if not.
*/
PUBLIC int BIND8_LEXPARSE::expect (const char *s)
{
	int ret = -1;
	const char *tok = gettoken();
	if (tok != NULL && strcmp(tok,s)==0){
		ret = 0;
	}else{
		error (MSG_U(E_EXPECTED,"Expected token %s"),s);
	}
	return ret;
}


