#include <string.h>
#include <misc.h>
#include "netconf.h"
#include "../paths.h"
#include "netconf.m"
#include <dialog.h>
#include "internal.h"

static NETCONF_HELP_FILE help_dev ("devlist");

static CONFIG_FILE f_dev (PROC_NET_DEV
	,help_dev
	,CONFIGF_PROBED);


/*
	Get the list of all network device (except aliases).
	Return -1 if any errors.
	Return the number of entry in list.
*/	
int devlist_read (SSTRINGS &list)
{
	int ret = -1;
	FILE *fin = f_dev.fopen ("r");
	if (fin != NULL){
		char buf[600];
		if (fgets(buf,sizeof(buf)-1,fin)!=NULL
			&& fgets(buf,sizeof(buf)-1,fin)!=NULL){
			while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
				char word[600];
				str_copyword (word,buf);
				char *pt = strchr(word,':');
				if (pt != NULL){
					if (strchr(pt+1,':')==NULL){
						/* #Specification: netconf / proc/net/dev
							netconf assume that /proc/net/dev has the following
							format. 2 lines of heading followed by device info.
							Each line begin by the device name. A device name
							is ended by :, while an alias has a : in the middle
							and at the end.
						*/
						*pt = '\0';
						list.add (new SSTRING(word));
					}
				}
			}
			ret = list.getnb();
		}
		fclose (fin);
	}
	return ret;
}


/*
	Return != 0 if a network device is available in the kernel

	This function is implemented by reading /proc/net/dev instead of
	asking the kernel directly. Asking the kernel directly has the side
	effect of triggering kerneld if the device is not loaded. This is
	not always what we want.
*/
int devlist_devexist (const char *devname)
{
	int ret = 0;
	SSTRINGS list;
	if (devlist_read(list)!=-1){
		int n = list.getnb();
		for (int i=0; i<n; i++){
			SSTRING *s = list.getitem(i);
			if (s->cmp(devname)==0){
				ret = 1;
				break;
			}
		}
	}
	return ret;
}

/*
	Setup a list of devices in a Combo field
*/
void devlist_setcombo (FIELD_COMBO *comb)
{
	static const char *tbinter[][2]={
		{"eth0",	MSG_U(F_FIRSTETH,"First ethernet adaptor")},
		{"eth1",	MSG_U(F_SECOND,"Second")},
		{"eth2",	MSG_U(F_THIRD,"Third")},
		{"eth3",	MSG_U(F_FOURTH,"Fourth")},
		{"arc0",	MSG_U(F_ARCNET,"Arcnet adaptor")},
		{"atp0",	MSG_U(F_ATPNET,"Pocket adaptor")},
		{"dummy0",	MSG_U(F_FIRSTDUM,"First dummy device")},
		{"dummy1",	MSG_R(F_SECOND)},
		{"ppp0",	MSG_U(F_FIRSTPPP,"First PPP channel")},
		{"ppp1",	MSG_R(F_SECOND)},
		{"sl0",		MSG_U(F_FIRSTSLIP,"First SLIP/CSLIP channel")},
		{"sl1",		MSG_R(F_SECOND)},
		{"tr0",		MSG_U(F_FIRSTTKR,"First Token ring adaptor")},
		{"tr1",		MSG_R(F_SECOND)},
	};
	for (unsigned int i=0; i<sizeof(tbinter)/sizeof(tbinter[0]); i++){
		comb->addopt (tbinter[i][0],tbinter[i][1]);
	}
}

